# -*- coding: utf-8 -*-
#
# This file is part of SENAITE.CORE
#
# Copyright 2018 by it's authors.
# Some rights reserved. See LICENSE.rst, CONTRIBUTORS.rst.

from Products.ATContentTypes.content import schemata
from Products.Archetypes import atapi
from bika.lims import bikaMessageFactory as _
from bika.lims.browser.bika_listing import BikaListingView
from bika.lims.config import PROJECTNAME
from bika.lims.interfaces import IWorksheetTemplates
from bika.lims.utils import get_link
from plone.app.content.browser.interfaces import IFolderContentsView
from plone.app.folder.folder import ATFolder, ATFolderSchema
from plone.app.layout.globals.interfaces import IViewView
from zope.interface.declarations import implements


class WorksheetTemplatesView(BikaListingView):
    implements(IFolderContentsView, IViewView)

    def __init__(self, context, request):
        super(WorksheetTemplatesView, self).__init__(context, request)
        self.catalog = 'bika_setup_catalog'
        self.contentFilter = {
            'portal_type': 'WorksheetTemplate',
            'sort_on': 'sortable_title'
        }
        self.context_actions = {
            _('Add'):
            {
                'url': 'createObject?type_name=WorksheetTemplate',
                'icon': '++resource++bika.lims.images/add.png'
            }
        }
        self.title = self.context.translate(_("Worksheet Templates"))
        self.form_id = "list_worksheettemplates"
        self.description = ""
        self.show_sort_column = False
        self.show_select_row = False
        self.show_select_column = True
        self.pagesize = 25
        self.icon = '{}/{}/{}'.format(
            self.portal_url,
            '++resource++bika.lims.images',
            'worksheettemplate_big.png'
        )
        self.columns = {
            'Title': {
                'title': _('Title'),
                'index': 'sortable_title'
            },
            'Description': {
                'title': _('Description'),
                'index': 'description',
                'toggle': True
            },
            'Instrument': {
                'title': _('Instrument'),
                'index': 'getInstrumentTitle',
                'toggle': True
            },
        }
        self.review_states = [
            {
                'id': 'default',
                'title': _('Active'),
                'contentFilter': {
                    'inactive_state': 'active'
                },
                'transitions': [
                    {
                        'id': 'deactivate'
                    },
                ],
                'columns': [
                    'Title',
                    'Description',
                    'Instrument'
                ]
            },
            {
                'id': 'inactive',
                'title': _('Dormant'),
                'contentFilter': {
                    'inactive_state': 'inactive'
                },
                'transitions': [
                    {
                        'id': 'activate'
                    },
                ],
                'columns': [
                    'Title',
                    'Description',
                    'Instrument'
                ]
            },
            {
                'id': 'all',
                'title': _('All'),
                'contentFilter': {},
                'columns': [
                    'Title',
                    'Description',
                    'Instrument'
                ]
            },
        ]

    def folderitem(self, obj, item, index):
        item['Description'] = obj.Description()
        item['replace']['Title'] = get_link(item['url'], item['Title'])
        item['Instrument'] = ''
        instrument = obj.getInstrument()
        item['Instrument'] = instrument and instrument.Title() or ''
        return item


schema = ATFolderSchema.copy()


class WorksheetTemplates(ATFolder):
    implements(IWorksheetTemplates)
    displayContentsTab = False
    schema = schema


schemata.finalizeATCTSchema(schema, folderish=True, moveDiscussion=False)
atapi.registerType(WorksheetTemplates, PROJECTNAME)
